from codap import spawn, Queue


class AsyncWriter:
    RUN = False
    def __init__(self, client):
        """Wraps the client connection with an async call"""
        self.q = Queue()
        self.client = client
        self.stop_q = Queue()

    def start(self):
        """Backgrounds the writing of data points"""
        self.RUN = True
        spawn(self._run)

    def stop(self):
        """Stop the writting of data points"""
        self.RUN = False
        self.q.put(None)
        self.stop_q.get()


    def write_points(self, body):
        """ Basically wraps the influxdb call """
        self.q.put_nowait(body)

    def _run(self):
        while self.RUN or not self.q.empty():
            data = self.q.get()
            if data is not None:
                self.client.write_points(data)

        self.stop_q.put(None)



import unittest


import asyncinflux


class MockInfluxClient:

    def __init__(self):
        self.data = None

    def write_points(self, points):
        if self.data is not None:
            self.data.extend(points)
        else:
            self.data = points

class TestAsycWriteClient(unittest.TestCase):

    def test_single(self):
        mic = MockInfluxClient()
        awc = asyncinflux.AsyncWriter(mic)
        awc.start()
        awc.write_points([{"foo": "bar"}])
        awc.stop()
        assert mic.data is not None, 'Expected mic.data not to be None'
        assert mic.data[0].has_key('foo'), 'Expected mic.data to have key "foo" however it is {0}'.format(mic.data)
        assert mic.data[0]["foo"] == "bar", 'Expected data["foo"] to be "bar" but was {0}'.format(mic.data['foo'])

    def test_multiple(self):
        mic = MockInfluxClient()
        awc = asyncinflux.AsyncWriter(mic)
        awc.write_points([{"foo": "bar1"}])
        awc.write_points([{"foo": "bar2"}])
        awc.start()
        awc.stop()

        assert mic.data is not None, 'Expected mic.data not to be None'
        assert mic.data[0].has_key('foo'), 'Expected mic.data[0] to have key "foo" however it is {0}'.format(mic.data[0])
        assert mic.data[0]['foo'] == "bar1", 'Expected data[0]["foo"] to be "bar1" but was {0}'.format(mic.data['foo'])
        assert mic.data[1]['foo'] == "bar2", 'Expected data[1]["foo"] to be "bar2" but was {0}'.format(mic.data['foo'])



from setuptools import setup


DESCRIPTION = """Uses codap to provide async influxdb"""
setup(
    name='asyncinflux',
    version='0.0.1',
    author='Lateef H. Jackson',
    author_email='lateef.jackson@gmail.com',
    description=(DESCRIPTION),
    license='Apache License V2',
    keywords='influxdb threading gevent eventlet coroutine',
    url='https://github.com/lateefj/asyncinflux',
    packages=['asyncinflux', 'tests'],
    install_requires=[
        'influxdb',
        'codap',
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Topic :: Utilities',
        'License :: OSI Approved :: BSD License',
    ],
)
